#! /usr/bin/python3

import argparse

import sys
import os
import csv
#import ipdb
import testeur
import pandas
import json
import re
import uuid
import time

# Configuration
default_host = "localhost"
default_user = "ubuntu"

dockers = {}

nbOK = 0
nbNOK = 0

def ok(msg):
    global nbOK
    
    nbOK=nbOK+1
    print ("\033[48;5;100m\033[1m{}\033[0m\t{}".format("OK", msg))

def nok(msg):
    global nbNOK
    
    nbNOK=nbNOK+1
    print ("\033[48;5;196m\033[1m\33[5m{}\033[0m\t{}".format("Not OK", msg))

def traite_ok_temp(vok, mess):
    if (vok == 1):
        ok(mess)
    else:
        nok(mess)

def affiche_results(results):
    for result in results:
        traite_ok_temp(result[0], result[1])

def get_descript(t, iddocker):
    """
    Récupère une descrition d'un docker dont on a le nom a priori a partir
    de la commande docker inspect
    """
    sortie, err = t.test("sudo docker inspect {}".format(iddocker))
    if len(err) != 0:
        raise ValueError("Le docker {} n'existe pas".format(iddocker))
    descr = {}
    inspect = json.loads("".join(sortie))[0]

    descr['Name'] = inspect['Name']
    descr['HostnamePath'] = inspect['HostnamePath']
    descr['HostsPath'] = inspect['HostsPath']

    descr['Volumes'] = inspect['Mounts']
    descr['Ports'] = inspect['NetworkSettings']['Ports']
    descr['Networks'] = {}
    for net in inspect['NetworkSettings']['Networks'].keys():
        descr['Networks'][net] = {}
        descr['Networks'][net]['IPAddress'] = inspect['NetworkSettings']['Networks'][net]['IPAddress']
        descr['Networks'][net]['Prefix'] = inspect['NetworkSettings']['Networks'][net]['IPPrefixLen']
        descr['Networks'][net]['Aliases'] = inspect['NetworkSettings']['Networks'][net]['Aliases']

    descr['Env'] = inspect['Config']['Env']
    descr['ExtraHosts'] = inspect['HostConfig']['ExtraHosts']
    return descr

def change_status_docker(nom, status, t):
    r,e = t.test("docker {} {}".format(status, nom))
    if (len(e) != 0):
        print("La commande docker {} {} a occasionner l'erreur {}".format(status, nom, '\n'.join(e)))


def existance_fichiers(t, fichiers):
    manque = []
    for fich in fichiers:
        if not os.path.exists(fich):
            manque.append(fich)
    return manque

def test_install_docker(t):
    """
    test l'installation de docker, si le docker démon fonctionne et si
    le proxy est bien mis
    """
    sortie, err = t.test("sudo docker ps -a")
    if len(err) != 0:
        print("Il y a une erreur pour la commande docker ps")
        for l in err:
            print("\t\"{}\"".format(l))
        nok("Docker a un problème")
    else:
        ok("Docker semble installé et fonctionnel")



def test_presence_docker(t, dat):
    """
    teste si les dockers décrit par dat existent
    
    effet de bords : stocke la description des dockers utilisés dans un tableau et lance eventuellement une commande pour allumer les docker

    """
    sortie, err = t.test("sudo docker ps -a", result="file")
    sort = pandas.read_fwf(sortie,header=0)

    
    print("Dockers trouvés :\n");
    for l in sort['NAMES']:
        print("\t\"{}\"".format(l))

    dockers = {}
    bon = True
    for regexnom in dat.keys():
        # le docker est-il dans la liste des noms
        trouv = sort[sort['NAMES'].map(lambda x: bool(re.match(regexnom,x)))]
#        trouv = sort['NAMES'][nom == sort['NAMES']]
        if len(trouv) == 1:
            pos = trouv.index[0]
            nom = str(trouv.loc[pos,'NAMES'])
            ok("Docker \"{}\" trouvé en position {}".format(nom, pos))
            if (dat[regexnom]['changeStatus'] is not None):
                change_status_docker(nom, dat[regexnom]['changeStatus'], t)
            dockers[dat[regexnom]['service']] = get_descript(t, nom)
            if re.match("^{}.*$".format(dat[regexnom]['image']),sort['IMAGE'][pos]):
                 ok("Le nom de l'image est bien \"{}\"".format(sort['IMAGE'][pos]))
            else:
                 bon = False
                 nok("Le nom de l'image est \"{}\" et pas \"{}\""
                     .format(sort['IMAGE'][pos], dat[regexnom]['image']))
        elif len(trouv) > 1:
            bon = False
            nok("\"{}\" dockers trouvés pour la regex {}".format(len(trouv), regexnom))
            nok("Donc l'image n'est pas bonne non plus")
        else:
            bon = False
            nok("Docker \"{}\" non trouvé".format(regexnom))
            nok("Donc l'image n'est pas bonne non plus")

    return dockers

def verif_partage(vol, source, dest, moderw):
    print("vérifie le partage de {} sur {} en mode RW {}"
          .format(vol['Source'], vol['Destination'], vol['RW']))
    return ((re.match(source,vol['Source']))
            and (re.match(dest,vol['Destination']))
            and (vol['RW'] == moderw))
    

def verif_venv(Venvs, valeur, messageOK, messageNOK):
    try:
        if (any(map(lambda x: re.match(valeur, x), Venvs))):
            ok(messageOK)
            return True
    except:
        print("{} n'est pas iterable".format(Venvs))
    
    nok(messageNOK)
    return False
        
def verif_ip(docker, reseau, ip):
    if reseau in docker['Networks'].keys():
        if (docker['Networks'][reseau]['IPAddress'] == ip):
            ok("{} a bien l'adresse {} dans le réseau {}"
               .format(docker['Name'], ip, reseau)
            )
            return True
        else:
            nok("{} a l'adresse {} au lieu de {} dans le réseau {}"
               .format(docker['Name'],
                       docker['Networks'][reseau]['IPAddress'], ip, reseau))
            return False
    else:
        nok("{} n'a pas d'adresse dans le réseau {}"
            .format(docker['Name'], reseau))
        return False

def verif_alias(descr, reseau, alias):
    """
    Vérifie la présence d'alias réseau
    
    @param descr : la description où chercher
    @param reseau, alias : le réseau et l'alias à chercher
    @return : un résultat de rechercher (nbpoint, message)
    """
    if not (reseau in descr['Networks'].keys()):
        return (0, "le réseau {} n'est pas dans {}".format(reseau, descr['Name']))    
    if (alias in descr['Networks'][reseau]['Aliases']):
        return (1, "le docker {} a bien l'alias {} est dans le réseau {}".format(descr['Name'], alias, reseau))
    else:
        return (0, "le docker {} n'a pas l'alias {} dans le réseau {}".format(descr['Name'], alias, reseau))

def test_docktest(docker, t):
    result, err = t.test("sudo cat '{}'".format(docker['HostnamePath']))
    if ((len(result)>0) and (result[0]=='test\n')) :
        ok("Ne hostname du docker est bien test")
    else:
        if (len(result)>0):
            nok("le hostname du docker est {} au lieu de test".format(result[0]))
        else:
            nok("le hostname du docker n'existe pas")

    if (any(map(lambda x: x=="MYSQL_ROOT_PASSWORD=passbdd", docker['Env']))):
        ok("on trouve la variable d'env pour le password de root")
    else:
        nok("Le mot de passe root ne semble pas bon")

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/datatest",
                              "/var/lib/mysql",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de données est trouvé")
    else:
        nok("PAs de partage pour les données")


    res, err = t.test('ls /home/ubuntu/docker/datatest/')
    if (any(map(lambda x: x=='MABASE\n', res))):
        ok("On trouve la base MABASE")
    else:
        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
        for f in res:
            print ("  - {}".format(f))
        nok("On ne trouve la base MABASE")

    res, err = t.test('ls /home/ubuntu/docker/datatest/')
    if (any(map(lambda x: x=='MABASE2\n', res))):
        ok("On trouve la base MABASE2")
    else:
        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
        for f in res:
            print ("  - {}".format(f))
        nok("On ne trouve la base MABASE2")

    return True

    
def test_dockbdd(docker, t):
    result, err = t.test("sudo cat '{}'".format(docker['HostnamePath']))
    if ((len(result)>0) and (result[0]=='bdd\n')) :
        ok("Ne hostname du docker est bien bdd")
    else:
        if (len(result)>0):
            nok("le hostname du docker est {} au lieu de bdd".format(result[0]))
        else:
            nok("le hostname du docker n'existe pas")
    print("Vérification des variables d'environement")
    venvs = [
        ["MYSQL_ROOT_PASSWORD=passbdd", "On trouve le mot de passe root", "Le mot de passe root n'est pas bon"],
        ["MYSQL_USER=owncloud","On trouve l'utilisateur owncloud", "On ne trouve pas l'utilisateur owncloud"],
        ["MYSQL_PASSWORD=passowc","MdP de owncloud", "mauvais MdP pour owncloud"],
        ["MYSQL_DATABASE=owncloud","La base de donnée", "Pas la base de donnée"]
    ]
    for ve in venvs:
        verif_venv(docker["Env"], ve[0], ve[1], ve[2])

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/data",
                              "/var/lib/mysql",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de données est trouvé")
    else:
        nok("Pas de partage pour les données")


    res, err = t.test('ls /home/ubuntu/docker/data/')
    if (any(map(lambda x: x=='owncloud\n', res))):
        ok("On trouve la base owncloud")
    else:
        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
        for f in res:
            print ("  - {}".format(f))
        nok("On ne trouve la base owncloud")

    verif_ip(docker, "interne", "172.18.100.10")
        
    return True
    
def test_dockpma(docker, t):

    print("Vérification des variables d'environement")
    venvs = [
        ["PMA_HOST=bdd", "La base est bien configurée", "La base de donnée n'est pas bien configurée le serveur doit être nomé 'bdd'"],
        ["PMA_ABSOLUTE_URI=(http://192.168.[0-9]+.[0-9]+)?/phpMyAdmin/?","L'url d'accès est bonne pour nginx", "L'url d'accès n'est pas bonne pour nginx"],
    ]
    for ve in venvs:
        verif_venv(docker["Env"], ve[0], ve[1], ve[2])

    verif_ip(docker, "interne", "172.18.100.11")
    verif_venv(docker["ExtraHosts"], "bdd:172.18.100.10",
               "La liaison avec la bdd est bonne",
               "La liaison avec la bdd n'est pas bonne"
    )

    res, err = t.test("wget --no-proxy http://172.18.100.11 -O -")
    if (any(map(lambda x: re.search("HTTP request sent, awaiting response... 200 OK",x), err))):
        ok("dockpma semble répondre")
    else:
        nok("dockpma ne semple pas répondre")

    bon = False
    for lig in res:
        m = re.search("PMA_commonParams", lig)
        if (m):
            ok("dockpma semble être un docker phpMyAdmin")
            m1 = re.search('rootPath:"([^"]*)"', lig)
            if (re.match("/?phpMyAdmin/?",m1.group(1))):
                ok("le rootpath est bien {}".format(m1.group(1)))
            else:
                nok("le rootpath est {}".format(m1.group(1)))
            bon=True
    if (not bon):
        nok("dockpma ne semble pas être un docker phpMyAdmin")
        
        
def test_reponse_owncloud(t, url, test_onlyoffice=False):

    
    
    nomfich = "/tmp/tempo_{}.txt".format(uuid.uuid4())

    results = []
    com = "wget -S --timeout=5 --connect-timeout=2 -t 1 --save-cookies {} --keep-session-cookies --no-proxy {} -O -".format(nomfich, url)
    print("commande {}".format(com))
    res, err = t.test(com)
    if (any(map(lambda x: re.search("HTTP/1.1 200 OK",x), err))):
        result = (1, "un serveur web semble répondre à l'adresse {}".format(url))
    else:
        result = (0, "Aucun serveur web ne semble répondre à l'adresse {} la réponse est {}".format(url, ' '.join(err)))
    results.append(result)

    # Recherche du request tocken
    rtocken = None
    foundowc = False
    for ligne in res:
        m = re.search('<head data-requesttoken="([^"]+)">', ligne)
        if (m):
            rtocken = m.group(1)
        m = re.search('owncloud\.org', ligne)
        if (m):
            foundowc = True
#    print(com)
#    print(' '.join(err))
#    print(' '.join(res))

    if (foundowc):
        if (rtocken is None):
            results.append((0, "{} ne semble pas un site owncloud, il n'y a pas de tocken".format(url)))
        else:
            results.append((1, "{} semble un site owncloud".format(url)))
    else:
        results.append((0, "{} ne semble pas un site owncloud".format(url)))

    if ((rtocken is None) or (not foundowc)):
        results = results + [(0, 'On ne peux pas tester la suite') for i in range(0,4)]

        affiche_results(results)
        return(results)

    time.sleep(0.1)
    com ="""
    wget -S --timeout=5 --connect-timeout=2 -t 1 --post-data="user=admin&password=passadmin&requesttoken={}" \
         --load-cookies {} --save-cookies {} --keep-session-cookies --no-proxy {} -O -
    """.format(rtocken, nomfich, nomfich, url+"login")
    print("commande {}".format(com))    
    res, err = t.test(com)
    if (any(map(lambda x: re.search("Location: http://[^/]+/apps/files/",x), err))):
        redirect = True
        result = (1, "Le login semble possible ici {}".format(url+"/login"))
    else:
        redirect = False
        result = (0, "Le login ne semble pas possible ici {} la réponse est {}".format(url+"/login", ' '.join(err)))
    results.append(result)
#    print(com)
#    print(' '.join(err))
#    print(' '.join(res))
    
    i = 0
    fpassword = False
    flogout = False
    logouturl = None
    fadm = False
    fcal = False
    for lig in res:
        m = re.search('<a id="logout" href="([^"]+)">', lig)
        if (m):            
            logouturl = m.group(1)
            if re.search(url, logouturl):
                print("L'url de logout est bizarre : {}".format(logouturl))
                flogout = True
        m = re.search('<input type="password" name="password" id="password" value=""',lig)
        if (m):
            fpassword = True
        m = re.search('<head data-user="admin" data-user-displayname="admin"',lig)
        if (m):
            fadm = True
        m = re.search('/custom/calendar/img/calendar.svg',lig)
        if (m):
            fcal = True
        i=i+1
#        if ((i<50) or (i>200)):
#            sys.stdout.write(lig)
        #      break
    
    if (fpassword and redirect):
        result = (0, "La connexion ne semble pas fonctionner")
    else:
        result = (1, "On semble pouvoir se connecter en tant qu'admin")
    results.append(result)
        
    if (fadm):
        result = (1, "On est bien connecté en tant qu'admin")
    else:
        result = (0, "Je ne trouve pas l'utilisateur admin")
    results.append(result)

    if (fcal):
        result = (1, "On trouve le module calendar")
    else:
        result = (0, "On ne trouve pas le module calendar")
    results.append(result)
    if (flogout):
        result = (1, "On peut se déconnecter")
    else:
        result = (0, "On ne peut pas se déconnecter l'url est {}".format(url))
    results.append(result)

    if (test_onlyoffice):
            com ="""
            wget -S  --timeout=5 --connect-timeout=2 -t 1 \
            --load-cookies {} --save-cookies {} --keep-session-cookies --no-proxy "{}" -O -
            """.format(nomfich, nomfich, url+"apps/files/?dir=/Documents&fileid=14")
            print("commande {}".format(com))
            res, err = t.test(com)
            if (any(map(lambda x: re.search("/custom/onlyoffice/",x), res))):
                result = (1, "Onlyoffice semble fonctionner (et on compte 3 fois)")
            else:
                result = (0, "Onlyoffice ne semble pas fonctionner (et on compte 3 fois)")
            # print("commande '{}'".format(com))
            # print("ERR#############################")
            # print(" ".join(err))
            # print("RES#############################")
            # print(" ".join(res))
            results.append(result)
            results.append(result)
            results.append(result)

    
    if (logouturl is not None):
        com = """
        wget -S --timeout=5 --connect-timeout=1 -t 2 --load-cookies {} --save-cookies {} --keep-session-cookies --no-proxy {} -O -
        """.format(nomfich, nomfich, logouturl)
        print("commande {}".format(com))
        res, err = t.test(com)
        if (any(map(lambda x: re.search("http://[^/]+/login",x), err))):
            resultign = (1, "La déconnexion semble possible ici {}".format(logouturl))
        else:
            resultign = (0, "La déconnexion ne semble pas possible ici {} la réponse est {}".format(logouturl, ' '.join(err)))
        print(resultign)
        #    print(" ".join(res))
        #        print(com)
        #        print(' '.join(err))
        #        print(' '.join(res))
    affiche_results(results)
    return(results)
    

def test_docknginx(docker, t):

    verif_ip(docker, "interne", "172.18.100.5")
    verif_venv(docker["ExtraHosts"], "(dock)?pma:172.18.100.11",
               "La liaison avec pma est bonne",
               "La liaison avec pma n'est pas bonne (la machine doit s'appeléer pma ou dockpma"
    )
    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/.*nginx\.conf",
                              "/etc/nginx/nginx.conf",
                              False
                ),
                docker['Volumes']))):
        ok("Le partage de configuration est trouvé")
    else:
        nok("Pas partage de configuration")

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/nginx/?",
                              "/www/?",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de site trouvé")
    else:
        nok("Pas partage de site")

    res, err = t.test("wget --no-proxy http://localhost/phpMyAdmin/ -O -")
    if (any(map(lambda x: re.search("HTTP request sent, awaiting response... 200 OK",x), err))):
        ok("pma semble répondre via nginx semble répondre")
    else:
        nok("dockpma ne semple pas répondre via nginx")

    bon = False
    for lig in res:
        m = re.search("PMA_commonParams", lig)
        if (m):
            ok("dockpma semble être un docker phpMyAdmin")
            m1 = re.search('rootPath:"([^"]*)"', lig)
            if (re.match("/?phpMyAdmin/?",m1.group(1))):
                ok("le rootpath est bien {}".format(m1.group(1)))
            else:
                nok("le rootpath est {}".format(m1.group(1)))
            bon=True
    if (not bon):
        nok("dockpma ne semble pas être un docker phpMyAdmin")

def nettoye_session_owc(t, dock):
    res,err = t.test("docker exec -t {} ls /mnt/data/sessions/".format(dock))
    if (len(err)>0):
        print("La commande ls a donné une erreur {}".format(' '.join(err)))
    for lig in res:
        lig = re.sub("\n", "", lig)
        lig = re.sub("\r", "", lig)
#        print("lig='{}'".format(lig))
        files = lig.split()
#        print(files)
        for file in files:
            if (file != ''):
                print("efface '{}'".format(file))
                com = "docker exec -t {} rm /mnt/data/sessions/{}".format(dock,file)
                res,err = t.test(com)
                if (len(err)>0):
                    print("La commande {} a donné une erreur {}".format(com, ' '.join(err)))


def test_dockowncloud(docker, ipVM, t):
    #nettoye_session_owc(t, docker['Name'])

    venvs = [
        ["OWNCLOUD_ADMIN_USERNAME=admin",
         "L'utilisateur admin est connu",
         "Il manque l'utilisateur admin"],
        ["OWNCLOUD_DB_HOST=bdd",
         "La base de donnée est configurée",
         "La base de donnée n'est pas configurée"],
    ]
    for ve in venvs:
        verif_venv(docker["Env"], ve[0], ve[1], ve[2])

    
    verif_ip(docker, "interne", "172.18.100.15")
    verif_venv(docker["ExtraHosts"], "bdd:172.18.100.10",
               "La liaison avec bdd est bonne",
               "La liaison avec bdd n'est pas bonne (la machine doit s'appeler bdd)"
    )

    
    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/files/?",
                              "/mnt/data/?",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de fichiers de données est trouvé")
    else:
        nok("Pas partage de fichiers de données")

    res, mess = verif_alias(docker, 'interne', 'owncloud')
    traite_ok_temp(res, mess)

    com = """
    docker exec -t dockowncloud \
           sudo -u www-data php occ app:list
    """
    res,err = t.test(com)
    if (len(err)>0):
        print("La commande {} a fait une errreur : {}".format(com, err))

    fcal = False
    for lig in res:
        #        sys.stdout.write(lig)
        if (re.search("- calendar",lig)):
            fcal=True
        if (re.search("^Disabled:",lig)):
            break
    if (fcal):
        ok("le module calendar a été trouvé et activé")
    else:
        nok("le module calendar n'est pas trouvé et activé")
    
    com = """
    docker exec -t dockowncloud \
           sudo -u www-data php occ user:lastseen admin
    """
    res,err = t.test(com)
    if (len(err)>0):
        print("La commande {} a fait une errreur : {}".format(com, err))

    dateladm = None
    for lig in res:
        #        sys.stdout.write(lig)
        m = re.search("admin.*login:(.*)",lig)
        if (m):
            dateladm = m.group(1)
    if (dateladm is not None):
        ok("l'admin s'est connecté le {}".format(dateladm))
    else:
        nok("l'admin ne s'est jamais connecté")

    #    test_reponse_owncloud(t, 'http://172.18.100.15:8080/')
    test_reponse_owncloud(t, 'http://{}/'.format(ipVM))


def verif_ping(t, dockfrom, to, expect):
    com = "docker exec -t {} ping -c 3 {}".format(dockfrom, to)
    res, err = t.test(com)
    if (len(err)>0):
        print("La commande {} stop à générée l'erreur {}"
              .format(com, " ".join(err)))
        return (0, "Impossible de faire un ping")
    address = None
    for lig in res:
        m = re.match(".*[0-9]+ bytes from ([0-9.]+): seq=[0-2] ttl=64 time=[0-9.]+ .*", lig)
        if (m):
            address = m.group(1)
            break
    if (address is None):
        return (0, "Aucun ping ne passe entre {} et {} cela donne {}".format(dockfrom, to, " ".join(res)))
    
    if (address == expect):
        return (1, "Un ping au moins fonctionne entre {} et {}".format(dockfrom, to))
    else:
        return (0, "Un ping au moins fonctionne entre {} et {} mais avec une mauvaise adresse".format(dockfrom, to))
        
def test_compose_aliases(t, descriptions):
    alias = [
        {'alias': "mysql", 'service': "mysql"},
        {'alias': "bdd",   'service': "mysql"},
        {'alias': "pma",   'service': "phpmyadmin"},
        {'alias': "owc",   'service': "owncloud"},
    ]

    if (('phpmyadmin' not in descriptions.keys())
        or ('compose_rescompose' not in descriptions['phpmyadmin']['Networks'].keys())):
        results = [(0, "impossible de tester l'alias '{}' car phpmyadmin ne peut pinguer".format(a['alias'])) for a in alias]
        affiche_results(results)
        return results

    results= []
    for a in alias:
        result = verif_ping(t,
                            descriptions['phpmyadmin']['Name'],
                            a['alias'],
                            descriptions[a['service']]['Networks']['compose_rescompose']['IPAddress'])
        traite_ok_temp(result[0], result[1])
        results.append(result)

    return results

                

def test_compose_owncloud(t, docker, ipVM):
    #nettoye_session_owc(t, docker['Name'])
    
    venvs = [
        ["OWNCLOUD_ADMIN_USERNAME=admin",
         "L'utilisateur admin est connu",
         "Il manque l'utilisateur admin"],
        ["OWNCLOUD_DB_HOST=bdd",
         "La base de donnée est configurée",
         "La base de donnée n'est pas configurée"],
    ]
    for ve in venvs:
        verif_venv(docker["Env"], ve[0], ve[1], ve[2])

    
    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/compose/data/owncloud/?",
                              "/mnt/data/?",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de configuration est trouvé")
    else:
        nok("Pas partage de configuration")

    res, mess = verif_alias(docker, 'compose_rescompose', 'owncloud')
    traite_ok_temp(res, mess)

    com = """
    docker exec -t {} \
           sudo -u www-data php occ app:list
    """.format(docker['Name'])
    
    res,err = t.test(com)
    if (len(err)>0):
        print("La commande {} a fait une errreur : {}".format(com, err))

    fcal = False
    foo  = False
    for lig in res:
        #        sys.stdout.write(lig)
        if (re.search("- calendar",lig)):
            fcal=True
        if (re.search("- onlyoffice",lig)):
            foo=True
        if (re.search("^Disabled:",lig)):
            break
    if (fcal):
        ok("le module calendar a été trouvé et activé")
    else:
        nok("le module calendar n'est pas trouvé et activé")
    if (foo):
        ok("le module onlyoffice a été trouvé et activé")
    else:
        nok("le module onlyoffice n'est pas trouvé et activé")

    com = """
    docker exec -t {} \
           sudo -u www-data php occ user:lastseen admin
    """.format(docker['Name'])
    
    res,err = t.test(com)
    if (len(err)>0):
        print("La commande {} a fait une errreur : {}".format(com, err))

    dateladm = None
    for lig in res:
        #        sys.stdout.write(lig)
        m = re.search("admin.*login:(.*)",lig)
        if (m):
            dateladm = m.group(1)
    if (dateladm is not None):
        ok("l'admin s'est connecté le {}".format(dateladm))
    else:
        nok("l'admin ne s'est jamais connecté")

    #    test_reponse_owncloud(t, 'http://172.18.100.15:8080/')
    test_reponse_owncloud(t, 'http://{}/'.format(ipVM), True)

        
def test_all(server="localhost", user=None, key=None, parts = ["debut", "owncloud"]):
    global nbOK
    global nbNOK
    
    nbOK=0
    nbNOK=0
    
    if server == "localhost":
        t = testeur.testeur()
    else:
        t = testeur.testeur_ssh(server, user, key=key)

    #    test_install_docker(t)

    if (('debut' in parts)
        or ('owncloud' in parts)):
        res, err = t.test("docker-compose -f /home/ubuntu/compose/docker-compose.yml stop")
        if (len(err)>0):
            print("La commande docker-compose stop à générée l'erreur {}"
                  .format(" ".join(err)))
        res, err = t.test("docker start dockbdd dockowncloud dockpma docknginx")
        if (len(err)>0):
            print("La commande docker stop start à générée l'erreur {}"
                  .format(" ".join(err))  )  
        
        dat = {
            'docktest'    : {'image': 'mysql:5.7.23'  , 'changeStatus':'stop'  , 'service': 'docktest'}, 
            'dockbdd'     : {'image': 'mysql:5.7.23'  , 'changeStatus':'start' , 'service': 'dockbdd'},
            'dockpma'     : {'image': 'phpmyadmin'    , 'changeStatus':'start' , 'service': 'dockpma'},
            'docknginx'   : {'image': 'nginx'         , 'changeStatus':'start' , 'service': 'docknginx'},
            'dockowncloud': {'image': 'owncloud-perso', 'changeStatus':'start' , 'service': 'dockowncloud'},
        }
    
        dockers = test_presence_docker(t, dat)
        #verifie_docker(t)

    if ('debut' in parts):
        print("####################################")
        print("## Vérification du docker dockest ##")
        print("####################################")
        if ('docktest' in dockers.keys()):
            test_docktest(dockers['docktest'], t)
        print("####################################")
        print("## Vérification du docker dockbdd ##")
        print("####################################")
        if ('dockbdd' in dockers.keys()):
            test_dockbdd(dockers['dockbdd'], t)
        print("####################################")
        print("## Vérification du docker dockpma ##")
        print("####################################")
        if ('dockpma' in dockers.keys()):
            test_dockpma(dockers['dockpma'], t)
        print("####################################")
        print("## Vérification du docker docknginx ##")
        print("####################################")
        if ('docknginx' in dockers.keys()):
            test_docknginx(dockers['docknginx'], t)

    if ('owncloud' in parts):
        print("#########################################")
        print("## Vérification du docker dockowncloud ##")
        print("#########################################")
        if ('dockowncloud' in dockers.keys()):
            test_dockowncloud(dockers['dockowncloud'], server, t)

    if ('compose' in parts):
        print("#########################################")
        print("## Vérification des compose ##")
        print("#########################################")
        res, err = t.test("docker stop docknginx dockbdd dockpma dockowncloud")
        if (len(err)>0):
            print("La commande docker stop start à générée l'erreur {}"
                  .format(" ".join(err))  )  
        res, err = t.test("docker-compose -f /home/ubuntu/compose/docker-compose.yml start")
        if (len(err)>0):
            print("La commande docker-compose start à générée l'erreur {}"
              .format(" ".join(err)))

        dat = {
            'compose_mysql_.*'      : {'image': 'mysql:5.7.23'    , 'changeStatus':None, 'service': 'mysql'}, 
            'compose_phpmyadmin_.*' : {'image': 'phpmyadmin'      , 'changeStatus':None, 'service': 'phpmyadmin'}, 
            'compose_nginx_.*'      : {'image': 'nginx'           , 'changeStatus':None, 'service': 'nginx'},
            'compose_owncloud_.*'   : {'image': 'compose_owncloud', 'changeStatus':None, 'service': 'owncloud'},
            'compose_onlyoffice_.*' : {'image': 'onlyoffice', 'changeStatus':None, 'service': 'onlyoffice'},
        }
        dockers = test_presence_docker(t, dat)
        test_compose_aliases(t, dockers)
        if ('owncloud' in dockers.keys()):
            test_compose_owncloud(t, dockers['owncloud'], server)
        
    t.close()
    return (nbOK,nbNOK)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Passe des scripts de test à distance ou en local")
    parser.add_argument("-s", "--server", type=str, dest="server", default=default_host, help="La machine sur laquelle se connecter")
    parser.add_argument("-u", "--user", type=str, dest="user",
                        default=default_user, help="Le nom de l'utilisateur pour se connecter sur la machine (n'est pas considéré pour la machine locale")
    parser.add_argument("-k", "--key", type=str, dest="key", default=None, help="nom du fichier de clef")

    parser.add_argument("parts", type=str, metavar="part", nargs='+', 
                        help="Parties de la vérification à faire debut, owncloud ou compose."
                            +"Il ne faut pas utiliser compose avec les autres")
    
    args = parser.parse_args()

    print("##################################")
    print("##################################")
    print("Test du serveur {} pour {}".format(args.server, " ".join(args.parts)))
    print("##################################")
    print("##################################")
    
    nbok, nbnok = test_all(args.server, args.user,args.key, args.parts)
    print ("## Au final {}/{}".format(nbOK, nbOK+nbNOK))
    
